/**************************************************************************************************/

static char *DecodeBytes(unsigned char *srce, unsigned long bytes, unsigned long depth, struct IconImage *im)
{
    /*************************************/
    /*    FUNCTION BASED ON DECODERLE    */
    /*************************************/

    //unsigned char *srce is the encoded bitstream of bytes 
    //unsigned long bytes is total bytes (width*height)
    //unsigned long depth is the depth of the image data (<=8)
    //struct IconImage *im is the struct to store output bytes
    //alternatively the function can return the dest byte array

    //there is a main while loop with two inner while loops.
    //the main while loop determines when to end the function.
    //the upper while loop inside the main while loop is only for when copy & loop = 0.
    //that is where we determine the RLE copy values & RLE loop values by using bit shifting & masking.
    //the lower while loop inside the main while loop is for when copy Or loop > 0, it works with either.
    //it processes the loop bytes or the copy bytes by using bit shifting & masking & writes them all to the output byte array.

    // >> is the arithmetic (or signed) right shift operator. //padded with most significant bit
    // >>> is the logical (or unsigned) right shift operator.
    // << is the left shift operator, and meets the needs of both logical and arithmetic shifts.      
        
    long bits;
    int i, idx, mask;    
    unsigned char val;
    unsigned char *dest;
    long out, copy, loop; 
    unsigned long bitbuf;   
    
    idx = 0;
    out = 0;
    copy = 0;
    loop = 0; 
    bits = 0;
    bitbuf = 0;
    
    dest = malloc(bytes);
    if (!dest) return NULL;
    
    mask = (1<<depth)-1;
    //numcolors=(1<<depth) 
    //mask = (numcolors - 1)
    //0xFF is the 8-bit mask (255)    
    
    while (out < bytes)
    {
        while (!copy && !loop)
        {            
            //val = srce[idx]; //load byte
            if(bits < 8)
            {
                bitbuf = (bitbuf<<8)|*(srce++); //load byte
                bits += 8; //bit index
            }            
            val = (bitbuf>>(bits-8))&0xFF;  //get byte with mask
            bits -= 8; //bit adjust                     

            if (val <= 127)
            {
                (copy = val + 1);
                idx++; //RLE control byte
            }
            else if (val > 128) 
            {
                loop = (256 - val +1 );
                idx++; //RLE control byte
            }
            else
            continue;            
        }

        while (copy || loop)
        {
            if (loop)
            {
                //val = srce[idx]; //load next byte
                if(bits < depth)
                {
                    bitbuf = (bitbuf<<8)|*(srce++); //load byte                                   
                    bits += 8; //bit index
                }                
                val = (bitbuf>>(bits-depth))&mask; //get byte with mask                

                for (i=0; i<loop; i++)
                {
                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                }
                idx++; //RLE loop byte 
                bits -= depth; //bit adjust               
                out += loop;                
                loop = 0;
            }
            if (copy)
            {
                for (i=0; i<copy; i++)
                {
                    //val = srce[idx]; //load single bytes
                    if(bits < depth)
                    {
                        bitbuf = (bitbuf<<8)|*(srce++); //load byte                                       
                        bits += 8; //bit index
                    }                
                    val = (bitbuf>>(bits-depth))&mask; //get byte with mask                    

                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                    idx++; //RLE copy bytes
                    bits -= depth; //bit adjust
                }
                out += copy;
                copy = 0;
            }
        }
    } //end main while

    //return out != bytes ? "error decoding iff icon data" : 0;
    return dest;
}

/**************************************************************************************************/

static char *DecodeRLE(unsigned char *srce, unsigned long bytes, struct IconImage *im)
{
    /**************************************/
    /*    FUNCTION BASED ON UNPACKBITS    */
    /**************************************/

    //https://en.wikipedia.org/wiki/PackBits


    //unsigned char *srce is the RLE encoded stream of bytes 
    //unsigned long bytes is total bytes (width*height)
    //note that the depth of the image data will now be 8 bits
    //struct IconImage *im is the struct to store output bytes
    //alternatively the function can return the dest byte array

    //there is a main while loop with two inner while loops.
    //the main while loop determines when to end the function.
    //the upper while loop inside the main while loop is only for when copy & loop = 0.
    //that is where we determine the RLE copy values & RLE loop values.
    //the lower while loop inside the main while loop is for when copy Or loop > 0, it works with either.
    //it processes the loop bytes or the copy bytes & writes them all to the output byte array.
    
    int i, idx;    
    unsigned char val;
    unsigned char *dest;
    long out, copy, loop;    
    
    idx = 0;
    out = 0;
    copy = 0;
    loop = 0;
    
    dest = malloc(bytes);
    if (!dest) return NULL;    
    
    while (out < bytes)
    {
        while (!copy && !loop)
        {            
            val = srce[idx]; //load byte
            if (val <= 127)
            {
                (copy = val + 1);
                idx++; //RLE control byte
            }
            else if (val > 128) 
            {
                loop = (256 - val +1 );
                idx++; //RLE control byte                
            }
            else
            continue;            
        }

        while (copy || loop)
        {
            if (loop)
            {
                val = srce[idx]; //load next byte
                for (i=0; i<loop; i++)
                {
                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                }
                idx++; //RLE loop byte                
                out += loop;
                loop = 0;
            }
            if (copy)
            {
                for (i=0; i<copy; i++)
                {
                    val = srce[idx]; //load single byte                                       
                    dest[out+i] = val;
                    //im->ImageData[out+i] = val;
                    idx++; //RLE copy byte
                }
                out += copy;
                copy = 0;
            }
        }
    } //end main while 

    //return out != bytes ? "error decoding iff icon data" : 0;
    return dest;
}

/**************************************************************************************************/